/*
 * カナ LCD シールド(π用)(i2c方式)の文字一覧を表示させる
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o kanalcd-char-table kanalcd-char-table.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <wiringPi.h>
#include <kanalcd.h>


// カナ LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KANALCD_I2C_ADDR  0x3C


// UTF-8 の文字列
uint8_t * char_table[] = {

// 半角文字

	"ASCII 1\n" " !\"#$%&'()*+,-./",
	"ASCII 2\n" "0123456789:;<=>?",
	"ASCII 3\n" "@ABCDEFGHIJKLMNO",
	"ASCII 4\n" "PQRSTUVWXYZ[\\]^_",  // バックスラッシュの代わりに円記号が表示されます
	"ASCII 5\n" "`abcdefghijklmno",
	"ASCII 6\n" "pqrstuvwxyz{|}~",  // チルダ(~)を指定すると代わりに -1乗文字が表示されます

	// ISO 8859-1 の一部のみ表示できます。一部は代理文字が表示されます。代理文字も無いものは'■'になります
	"U+00A0\n" "¡¢£¤¥¦§¨©ª«¬­®¯",  // エディタによってはソフトハイフンが表示されない場合があります
	"U+00B0\n" "°±²³´µ¶·¸¹º»¼½¾¿",
	"U+00C0\n" "ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏ",
	"U+00D0\n" "ÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞß",
	"U+00E0\n" "àáâãäåæçèéêëìíîï",
	"U+00F0\n" "ðñòóôõö÷øùúûüýþÿ",

	"ｶﾅ1\n" "｡｢｣､･ｦｧｨｩｪｫｬｭｮｯ",
	"ｶﾅ2\n" "ｰｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿ",
	"ｶﾅ3\n" "ﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏ",
	"ｶﾅ4\n" "ﾐﾑﾒﾓﾔﾕﾖﾗﾘﾙﾚﾛﾜﾝﾞﾟ",

// 全角文字

	"ゼンカク1\n" "　！”＃＄％＆’（）＊＋，－．／",
	"ゼンカク2\n" "０１２３４５６７８９：；＜＝＞？",  // 全角文字の１～８を指定すると、ユーザー定義文字1～8 が表示されます
	"ゼンカク3\n" "＠ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯ",
	"ゼンカク4\n" "ＰＱＲＳＴＵＶＷＸＹＺ［￥］＾＿",
	"ゼンカク5\n" "‘ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏ",
	"ゼンカク6\n" "ｐｑｒｓｔｕｖｗｘｙｚ｛｜｝～",  // 全角文字のｘはバー付きで表示されます。～は'■'になります

	"わきごう\n" "ー、。・「」『』",

	"ひらがな1\n"  "ぁぃぅぇぉゃゅょっ",
	"ひらがな2\n"  "あいうえおかきくけこ",
	"ひらがな3\n"  "さしすせそたちつてと",
	"ひらがな4\n"  "なにぬねのはひふへほ",
	"ひらがな5\n"  "まみむめもやゆよ",
	"ひらがな6\n"  "らりるれろわをん",
	"ひらがな7\n"  "がぎぐげご",
	"ひらがな8\n"  "ざじずぜぞ",
	"ひらがな9\n"  "だぢづでど",
	"ひらがな10\n" "ばびぶべぼ",
	"ひらがな11\n" "ぱぴぷぺぽ",

	"カタカナ1\n"  "ァィゥェォャュョッ",
	"カタカナ2\n"  "アイウエオカキクケコ",
	"カタカナ3\n"  "サシスセソタチツテト",
	"カタカナ4\n"  "ナニヌネノハヒフヘホ",
	"カタカナ5\n"  "マミムメモヤユヨ",
	"カタカナ6\n"  "ラリルレロワヲン",
	"カタカナ7\n"  "ガギグゲゴ",
	"カタカナ8\n"  "ザジズゼゾ",
	"カタカナ9\n"  "ダヂヅデド",
	"カタカナ10\n" "バビブベボ",
	"カタカナ11\n" "パピプペポ",

	"ラテン+ギリシア\n" "äñöüαβεθμπρσΣΩ",

	"きごう\n"   "￠￡×÷√∞□■→←ｘ",  // 全角文字のｘはバー付きで表示されます
	"カンジ\n"   "千万円",

	"ユーザーていぎ\n" "１２３４５６７８①②③④⑤⑥⑦⑧",  // 全角文字の１～８か、①～⑧を指定すると、ユーザー定義文字1～8 が表示されます
};

int char_table_size = sizeof(char_table) / sizeof(*char_table);


int main (void)
{
	int fd, n;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// カナ LCD シールド(π用)の初期化
	fd = kanalcdInit (KANALCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}

	// 文字表を順番に表示させる
	for (n = 0; n < char_table_size; n++) {
		// 画面クリア
		kanalcdClear (fd);

		// 文字列をカナ LCD シールド(π用)に転送する
		kanalcdPuts (fd, char_table[n]);

//		// 適当な待ち時間を入れる。
//		delay (2000);

		getchar();  // Enterキー入力待ち
	}

	return 0;
}

