/*
 * カナ LCD シールド(π用)(i2c方式)の各種制御テスト
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o kanalcd-ctrl-test kanalcd-ctrl-test.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <stdio.h>
#include <wiringPi.h>
#include <kanalcd.h>


// カナ LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KANALCD_I2C_ADDR  0x3C



// ユーザー定義フォント
unsigned char font_data[8] = {
	0b10101, 0b01010, 0b10101, 0b01010, 0b10101, 0b01010, 0b10101, 0b01010,  // チェックパターン
};



int main (void)
{
	int fd;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// カナ LCD シールド(π用)の初期化
	fd = kanalcdInit (KANALCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}


	// テスト用の文字列を表示させる。初期化後は ⑧ はハート記号になっている。
	kanalcdPuts (fd, "<Testひょうじ>\nABCabc123アイウ⑧");


// 画面表示 ON/OFF

	printf ("画面表示OFF\n");
	getchar();  // Enterキー入力待ち

	kanalcdDisplay (fd, KANALCD_STATE_OFF);  // 画面表示を OFF

	printf ("画面表示ON\n");
	getchar();  // Enterキー入力待ち

	kanalcdDisplay (fd, KANALCD_STATE_ON);  // 画面表示を ON


// カーソル表示 ON/OFF

	printf ("カーソル表示ON\n");
	getchar();  // Enterキー入力待ち

	kanalcdCursor (fd, KANALCD_STATE_ON);  // カーソル表示を ON

	printf ("カーソル表示OFF\n");
	getchar();  // Enterキー入力待ち

	kanalcdCursor (fd, KANALCD_STATE_OFF);  // カーソル表示を OFF


// カーソル点滅 ON/OFF

	printf ("カーソル点滅ON\n");
	getchar();  // Enterキー入力待ち

	kanalcdCursorBlink (fd, KANALCD_STATE_ON);  // カーソル点滅を ON

	printf ("カーソル点滅OFF\n");
	getchar();  // Enterキー入力待ち

	kanalcdCursorBlink (fd, KANALCD_STATE_OFF);  // カーソル点滅を OFF

	printf ("再度、カーソル点滅ON\n");
	getchar();  // Enterキー入力待ち

	kanalcdCursorBlink (fd, KANALCD_STATE_ON);  // 以後、分かりやすいように、再度、カーソル点滅を ON


// Entry Mode Set

	printf ("逆順に文字を表示\n");
	getchar();  // Enterキー入力待ち

	kanalcdEntryModeSet (fd, KANALCD_EMS_SHIFT_OFF + KANALCD_EMS_AC_DEC);  // I/D=DEC, S=OFF

	kanalcdPuts (fd, "Reverse");  // 逆順に文字を表示。'esreveR' と表示される


	printf ("正順で文字を表示\n");
	getchar();  // Enterキー入力待ち

	kanalcdEntryModeSet (fd, KANALCD_EMS_SHIFT_OFF + KANALCD_EMS_AC_INC);  // I/D=INC, S=OFF

	kanalcdPuts (fd, "789千万円⑧");  // 正順で文字を表示。先ほどの 'esreve' が上書きされる。'R' だけ残る


	// デフォルトの EntryModeSet に戻す
	kanalcdEntryModeSet (fd, KANALCD_EMS_SHIFT_OFF + KANALCD_EMS_AC_INC);  // I/D=INC, S=OFF


// カーソルシフト

	printf ("カーソルを左シフト#1\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_CURSOR_LEFT);  // カーソルを左に１つシフト

	printf ("カーソルを左シフト#2\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_CURSOR_LEFT);  // カーソルを左に１つシフト

	printf ("カーソルを左シフト#3\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_CURSOR_LEFT);  // カーソルを左に１つシフト

	printf ("カーソルを左シフト#4\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_CURSOR_LEFT);  // カーソルを左に１つシフト

	printf ("上書き\n");
	getchar();  // Enterキー入力待ち

	kanalcdPuts (fd, "シフト");  // '千万円' を 'シフト' に上書きする


// 画面シフト

	printf ("画面を左シフト#1\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_DISPLAY_LEFT);  // 画面を左に１つシフト

	printf ("画面を左シフト#2\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_DISPLAY_LEFT);  // もう一回、画面を左に１つシフト

	printf ("画面を右シフト#1\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_DISPLAY_RIGHT);  // 画面を右に１つシフト

	printf ("画面を右シフト#2\n");
	getchar();  // Enterキー入力待ち

	kanalcdShift (fd, KANALCD_SHIFT_DISPLAY_RIGHT);  // もう一回、画面を右に１つシフト


// SD1602H を home return させる

	printf ("カーソルを home 位置に移動\n");
	getchar();  // Enterキー入力待ち

	kanalcdHome (fd);  // カーソルを home 位置に移動。カーソルは home 位置で点滅するはず


// ユーザー定義フォント

	printf ("ユーザー定義フォント変更\n");
	getchar();  // Enterキー入力待ち

	kanalcdCharDef (fd, 7, font_data);  // ユーザー定義フォントの８番目を、ハート記号からチェックパターンに変更する


// Backlight ソフト ON/OFF

	printf ("Backlight ソフト ON  ※ DIPSW3 の3番目(強制Backlight ON)はOFFにしておいてください\n");
	getchar();  // Enterキー入力待ち

	kanalcdBacklight (fd, KANALCD_STATE_ON);  // Backlight ソフト ON


	printf ("Backlight ソフト OFF\n");
	getchar();  // Enterキー入力待ち

	kanalcdBacklight (fd, KANALCD_STATE_OFF);  // Backlight ソフト OFF


// 画面クリア

	printf ("画面クリア\n");
	getchar();  // Enterキー入力待ち

	kanalcdClear (fd);  // 画面クリア


	return 0;
}

