/*
 * カナ LCD シールド(π用)(i2c方式): PWM Test
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o kanalcd-pwm-test kanalcd-pwm-test.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <wiringPi.h>
#include <kanalcd.h>


// 注意：本シールドの汎用出力ピンは +5V 仕様です。


// カナ LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KANALCD_I2C_ADDR  0x3C


int main (void)
{
	int fd, n;
	unsigned int pwm;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// カナ LCD シールド(π用)の初期化
	fd = kanalcdInit (KANALCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}

	// PWM 出力モードにする前に、初期値を設定しておきます。この時点では、まだ PWM 出力されません。
	kanalcdPwmWrite (fd, KANALCD_PIN_PWM0, 0);   // PWM0 に PWM 値を設定する
	kanalcdPwmWrite (fd, KANALCD_PIN_PWM1, 0);   // PWM1 に PWM 値を設定する
	kanalcdPwmWrite (fd, KANALCD_PIN_OUT0, 0);   // OUT0 に PWM 値を設定する

	// ピンモードを PWM 出力モードにします。ここで初めて PWM 出力が開始します。
	kanalcdPinMode (fd, KANALCD_PIN_PWM0, PWM_OUTPUT);  // PWM0 ピンを PWM 出力モードにする
	kanalcdPinMode (fd, KANALCD_PIN_PWM1, PWM_OUTPUT);  // PWM1 ピンを PWM 出力モードにする
	kanalcdPinMode (fd, KANALCD_PIN_OUT0, PWM_OUTPUT);  // OUT0 ピンを PWM 出力モードにする

	// 10回、繰り返します
	for (n = 0; n < 10; n++) {
		// 0～255 の範囲を順次、出力します
		for (pwm = 0; pwm <= 255; pwm++) {
			kanalcdPwmWrite (fd, KANALCD_PIN_PWM0, pwm);   // PWM0 に PWM 値を出力する
			kanalcdPwmWrite (fd, KANALCD_PIN_PWM1, pwm);   // PWM1 に PWM 値を出力する
			kanalcdPwmWrite (fd, KANALCD_PIN_OUT0, pwm);   // OUT0 に PWM 値を出力する
			delay (50);
		}
	}

	// ピンモードを、デジタル出力モードに戻します。PWM 出力が停止し、通常の ON/OFF 状態に戻ります。
	kanalcdPinMode (fd, KANALCD_PIN_PWM0, OUTPUT);  // PWM0 ピンを デジタル出力モードにする
	kanalcdPinMode (fd, KANALCD_PIN_PWM1, OUTPUT);  // PWM1 ピンを デジタル出力モードにする
	kanalcdPinMode (fd, KANALCD_PIN_OUT0, OUTPUT);  // OUT0 ピンを デジタル出力モードにする

	return 0;
}

