/*
 * 漢字 LCD シールド(π用)(i2c方式)の各種制御テスト
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o knjlcd-ctrl-test knjlcd-ctrl-test.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <stdio.h>
#include <wiringPi.h>
#include <knjlcd.h>


// 漢字 LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KNJLCD_I2C_ADDR  0x3C



int main (void)
{
	int fd;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// 漢字 LCD シールド(π用)の初期化
	fd = knjlcdInit (KNJLCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}


	// テスト用の文字列を表示させます。
	knjlcdPuts (fd, "≪Test表示≫\nABC123ｱｲｳ漢字");


// 画面表示 ON/OFF

	printf ("画面表示OFF\n");
	getchar();  // Enterキー入力待ち

	knjlcdDisplay (fd, KNJLCD_STATE_OFF);  // 画面表示を OFF

	printf ("画面表示ON\n");
	getchar();  // Enterキー入力待ち

	knjlcdDisplay (fd, KNJLCD_STATE_ON);  // 画面表示を ON


// カーソル表示 ON/OFF

	printf ("カーソル表示ON\n");
	getchar();  // Enterキー入力待ち

	knjlcdCursor (fd, KNJLCD_STATE_ON);  // カーソル表示を ON

	printf ("カーソル表示OFF\n");
	getchar();  // Enterキー入力待ち

	knjlcdCursor (fd, KNJLCD_STATE_OFF);  // カーソル表示を OFF


// カーソル点滅 ON/OFF

	printf ("カーソル点滅ON\n");
	getchar();  // Enterキー入力待ち

	knjlcdCursorBlink (fd, KNJLCD_STATE_ON);  // カーソル点滅を ON

	printf ("カーソル点滅OFF\n");
	getchar();  // Enterキー入力待ち

	knjlcdCursorBlink (fd, KNJLCD_STATE_OFF);  // カーソル点滅を OFF

	printf ("再度、カーソル点滅ON\n");
	getchar();  // Enterキー入力待ち

	knjlcdCursorBlink (fd, KNJLCD_STATE_ON);  // 以後、分かりやすいように、再度、カーソル点滅を ON


// SG12832A を home return させる

	printf ("カーソルを home 位置に移動\n");
	getchar();  // Enterキー入力待ち

	knjlcdHome (fd);  // カーソルを home 位置に移動。カーソルは home 位置で点滅するはず


// Backlight ソフト ON/OFF

	printf ("Backlight ソフト ON  ※ DIPSW3 の3番目(強制Backlight ON)はOFFにしておいてください\n");
	getchar();  // Enterキー入力待ち

	knjlcdBacklight (fd, KNJLCD_STATE_ON);  // Backlight ソフト ON


	printf ("Backlight ソフト OFF\n");
	getchar();  // Enterキー入力待ち

	knjlcdBacklight (fd, KNJLCD_STATE_OFF);  // Backlight ソフト OFF


// 画面クリア

	printf ("画面クリア\n");
	getchar();  // Enterキー入力待ち

	knjlcdClear (fd);  // 画面クリア


	return 0;
}

