/*
 * 漢字 LCD シールド(π用)(i2c方式): Digital Output Test
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o knjlcd-output-test knjlcd-output-test.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <wiringPi.h>
#include <knjlcd.h>


// 注意：本シールドの汎用出力ピンは +5V 仕様です。


// 漢字 LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KNJLCD_I2C_ADDR  0x3C


int main (void)
{
	int fd, n;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// 漢字 LCD シールド(π用)の初期化
	fd = knjlcdInit (KNJLCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}

	// PWM0⇒PWM1⇒OUT0⇒OUT1 点滅を、10回、繰り返します
	for (n = 0; n < 10; n++) {
		knjlcdDigitalWrite (fd, KNJLCD_PIN_PWM0, KNJLCD_STATE_ON);   // PWM0 出力を ON  にする
		delay (1000);
		knjlcdDigitalWrite (fd, KNJLCD_PIN_PWM0, KNJLCD_STATE_OFF);  // PWM0 出力を OFF にする

		knjlcdDigitalWrite (fd, KNJLCD_PIN_PWM1, KNJLCD_STATE_ON);   // PWM1 出力を ON  にする
		delay (1000);
		knjlcdDigitalWrite (fd, KNJLCD_PIN_PWM1, KNJLCD_STATE_OFF);  // PWM1 出力を OFF にする

		knjlcdDigitalWrite (fd, KNJLCD_PIN_OUT0, KNJLCD_STATE_ON);   // OUT0 出力を ON  にする
		delay (1000);
		knjlcdDigitalWrite (fd, KNJLCD_PIN_OUT0, KNJLCD_STATE_OFF);  // OUT0 出力を OFF にする

		knjlcdDigitalWrite (fd, KNJLCD_PIN_OUT1, KNJLCD_STATE_ON);   // OUT1 出力を ON  にする
		delay (1000);
		knjlcdDigitalWrite (fd, KNJLCD_PIN_OUT1, KNJLCD_STATE_OFF);  // OUT1 出力を OFF にする
	}

	return 0;
}

