/*
 * 漢字 LCD シールド(π用)(i2c方式)のテキスト表示デモ
 *
 * 使用ライブラリ: wiringPi, wiringPiDev
 * コンパイル方法: cc -o knjlcd-text-demo1 knjlcd-text-demo1.c -lwiringPi -lwiringPiDev
 */
#include <stdint.h>
#include <wiringPi.h>
#include <knjlcd.h>


// 漢字 LCD シールド(π用)の現在の i2c アドレス設定に合わせてください。
#define KNJLCD_I2C_ADDR  0x3C


// デモ表示用メッセージ文字列の配列
// UTF-8 の文字列
uint8_t * char_table[] = {

  "♪漢字LCDｼｰﾙﾄﾞ♫" "全角８文字×２行", // オープニング・メッセージ
  "組立ｷｯﾄ･要半田付" "✓価格￥4,200+税", // やはり漢字だと、工夫次第でメッセージを詰め込められる。アルファベットだと、こうはいかない。
  "☆JIS 第一水準字" "utf8対応,i2c通信", // JIS X 0208:1990 の第一水準 (非漢字＋漢字) は完全サポート
  "半角 157字、及び" "全角3737字に対応", // サポート字数 ASCII:94字＋半角カナ:63字、第一水準非漢字:524字＋第一水準漢字:2965字＋NEC特殊文字:73字＋第三水準非漢字(極一部のみ):175字
  "JIS X 0201片仮名" "所謂半角ｶﾅも対応", // 表示領域の狭さから半角カナは必要と判断。ISO 8859-1 も半角でサポートしたかったが、そうすると"§±¶×÷"の扱いが困難になる為、断念。極一部を全角で対応したのみ。
  "第二水準字は無し" "ｽｯﾊﾟﾘ 割切るべし", // 第二水準漢字は非サポート：代わりに大きい×印が表示される。
  "▼温度＝78.9℃▲" "π≒3.14159265…", // 表示例1：オーソドックスな使用方法
  "寿限無、寿限無、五劫の擦り切れ。",    // 表示例2："劫"って第一水準漢字だったとは…
  "NEC 特殊文字もOK" "①②③㎡№㍼㍻㈱", // 表示例3：ほとんどの方が第一水準字だと勘違いしていると思う。元来、機種依存文字だったが、2000年に第三水準に含まれる様になった。
  "☞JIS 第三水準の極一部記号に対応",    // JIS X 0213:2000 の利用度が高そうな 248 字の非漢字のみサポート：残念ながら非漢字全部 (659字) は無理だった。
  "€½⇄¼ℓ©☎⏎♠♡♢♣☀☁☂☃",    // こんな感じ
  "☗３三龍、王手！" "☖同桂☗待った！", // 待った無し
  "ｱｲﾇ 語も表示可能" "アイヌ・イタㇰ",   // アイヌ語は未勉強なので、Wikipedia からコピペ
  "半濁点付き片仮名" "カ゚～コ゚,セ゚ツ゚ト゚ㇷ゚", // JIS X 0213 の拡張カタカナの一部。この９字は Unicode では要文字合成だが、ちゃんと表示する様にした。なお、拡張ひらがなは非サポート。
  "i2cｱﾄﾞﾚｽ=0x0C/1C/2C/3Cより選択可",    // i2c(TWI) アドレス：0x0C～0x0F, 0x1C～0x1F, 0x2C～0x2F, 0x3C～0x3F のいずれかを DIP SW で設定しておく
  "但し、４連続アドレスを占有します",    // 例えば 0x3C を選んだら、0x3C,0x3D,0x3E,0x3F の４連続アドレスを占有してしまいます。

  "※次㌻以降の創作文では全ｶﾅ文字と",
  "※基本的な句読記号が使われて〼。", // 枡記号を使いたかっただけ

// 以降は「組見本用創作文『問わずがたりの洋酒外史』JIS 第１水準バージョン」です。
// ひらがな・カタカナの全部と、基本的な句読記号を使用している文章、だそうです。
//   ⇒ http://www.type-labo.jp/KumimihonREAD.html
// ※表示領域の狭さから、読み易いように空白・改行等を調整していますが、文章の内容そのものは改変していません。
// ※プログラムで禁則処理を書くのが面倒くさかったので、手作業で禁則処理してしまいました。(所々 '\n' が入っているのは、そのせい)

  "『問わずがたりの洋酒外史』佐藤章",
  "Copyright(C)1990 by Akira Satoh",
  "[Directed by TYPE-LABO & NAVEL]",  // "and" だと入りきらないので "&" に変更
  "洋酒といえば、誰でも最初に思い浮",
  "かべるのがウイスキー。いわば洋酒",
  "のシンボル的な存在なのだが、英語",
  "表記が［一般に〔米〕ではWhiskey,",
  "〔英〕では Whisky.］であることは",
  "あまり知られていない。米英両国で",
  "は、このスペルの差で自国産と輸入",
  "品を区別しているという。わが和製",
  "ウイスキーの“Whisky”という英国",
  "式表示は、手本にしたスコッチの",
  "フォルムに倣ったものであり、それ", // 'ォ' が行頭に来れないので、禁則処理させた
  "以上の意味はないようだ。",
  "カナ表記にして\nも、ごくまれに",   // '〈' が行末に来れないので、禁則処理させた
  "〈ウヰスキー〉という書き方を見か",
  "けるが、これとて差別化を意図した",
  "ものではなく単にカナづかいの時代",
  "性にすぎない。",
  "現在、カナ表記は〈ウイスキー〉に",
  "一本化しており、そこに国産・舶来",
  "の区別はない。最近は価格面での差",
  "もちぢまり、20年前に￥10,000だっ",
  "た本場のスコッチが半値近くになる",
  "など、ウイスキーは一層ポピュラー",
  "なものになってきた。しかし、消費",
  "量についていえ\nば、洋酒のトップ", // '、' が行頭に来れないので、禁則処理させた
  "は何といっても\nビールだろう。",
  "よっぽどの下戸はともかく、まず",
  "誰でもタシナミ程度には口にする。",
  "そのビールも明治中期には、せいぜ",
  "い好事家が味わう程度だったらし",
  "い。当時流行した川上音二郎の",     // '。' が行頭に来れないので、禁則処理させた
  "《おっぺけぺ節》では、まるでゲテ",
  "モノ扱い。「むやみに西洋を鼻にか",
  "け／日本酒なんぞは飲まれない／",
  "ビールにブラン\nデーベルモット／", // 'ー' が行頭に来れないので、禁則処理させた
  "腹にも馴れない洋食を／やたらに喰",
  "うのも負け惜しみ／ないしょで後架",
  "でヘドついて／まじめな顔してコー",
  "ヒのむ」と、特権階級をギューの目",
  "にあわせるネタにしている。観客も",
  "「ざまぁみやがれ！」とばかり同調",
  "して西洋カブレをオチョクり、日々",
  "のうっぷんを晴らしていた。",
  "もっとも、オッペケペッポーの攻勢",
  "に、ぐぅのネを上げた紳士や淑女が",
  "いたかどうか…西洋カブレとは、要",
  "するに欧米文化への信奉者のこと。",
  "〈鰯ノ頭ヲ信心〉する者があるなら",
  "ヨーロッパやアメリカの文物を信仰",
  "する者がいても一向に不思議はな",
  "い。信ずる者の強さで、依然グラス",
  "片手に舞踏会、アン・ドゥ・トロヮ",
  "とやっていたように思われる。",
  "憶測になるが、むしろ辛かったのは",
  "庶民の中の飲ん\nべぇ連中ではな",
  "かったか。ハイカラ族にユサぶりを",
  "かけた手前、泰西の酒には手を出し",
  "づらい。「なに、ビールだとぉ、べ",
  "らぼーめ、こちとら人間サマでぃ、",
  "カニじゃあるめぇしアブクなんか飲",
  "めるかってんだ」などと強がりなが",
  "らも、サゾ飲みたかったろうという",
  "気がする。",
  "しかし明治32年\n７月４日、恵比寿",
  "ビールのＰＲを目的にしたビアホー",
  "ルが新橋に登場。ジャムパン１ヶ",
  "２銭の時代に0.25リットル５銭で",
  "ビールの１ぱい売りを始めたことか",
  "ら様相は一転す\nる。レンガづくり",
  "のモダンな店ながら、はっぴ姿の車",
  "夫も金バッヂの紳士もゼニを払えば",
  "平等の客。誰はゞかることなくビー",
  "ルをグィとやって舌ツヾミを打て",
  "る。当初から評判になって人々が",
  "ワッと集まり、\nファンの数もふえ",
  "た。２ヵ月後の９月には中央新聞が",
  "その盛況ぶりを図入りで伝えたほど",
  "だった。",
  "この年は西暦1899年。まさに19世紀",
  "の〆くくりの年にビールのヌーヴェ",
  "ルバーグがぼっこうしたことにな",
  "る。恵比寿は文字通り“福の神”と",
  "なったわけだが、そのころは仮名表",
  "記も〈えびす〉・〈エビス〉ならぬ",
  "〈ゑびす〉・〈ヱビス〉で、いかに",
  "もゴリヤクのありそうな字ヅラに",
  "なっていた。",
  "20世紀に入って\n1906年になると、",
  "夏目ｿｳ石は小説\n『二百十日』（中", // '漱' は第二水準漢字なので、漢字 LCD シールドでは表示できないので、半角カナに変更しました
  "央公論・明治39年10月号）の中で次",
  "のようなエピソードを創作した。舞",
  "台は阿蘇のひなびた温泉宿。",
  "「ビールは御坐りまつせん」",       // ここから"旧かなづかい"っぽくなる
  "「ビールがない？───君ビールは",
  "ないとさ。何だか日本の領地でない",
  "様な気がする。情ない所だ」",
  "「なければ、飲まなくつても、いゝ",
  "さ」と圭さんは又泰然たる挨拶をす",
  "る。",
  "「ビールは御坐りませんばつてん、",
  "恵比寿なら御坐ります」",
  "「ハヽヽヽ愈(ｲﾖｲﾖ)妙になつて来",   // "(ｲﾖｲﾖ)" の部分は原文に無し。ルビの代わりに追加してみた。つーか '愈'自体、辞書引かなければ読めんかった(笑)
  "た。おい君ビールでない恵比寿があ",
  "るつて云ふんだ\nが、その恵比寿で",
  "も飲んで見るか\nね」",
  "「うん、飲んでもいゝ。───その",
  "恵比寿は矢つ張り瓶に這入つてるん",
  "だらうね、姉さ\nん」と圭さんは此",
  "時漸く下女に話かけた。",
  "ゆ上がりの男２人と、着物を「つん",
  "つるてんに着てゐる」下女のやりと",
  "りは、いささかマンザイのボケと",
  "ツッコミのセリフを思わせるが、",
  "ビールもこの時期には日本のスミズ",
  "ミまでじゅうぶんに出回っていたの",
  "がわかる。",
  "こうして19〜20世紀初頭に日本人に",
  "受け入れられた\nビールは、いまで",
  "は日常生活にすっかりとけこんでい",
  "る。それだけに洋酒という感じがう",
  "すい。豚カツが\n〈和食〉でカステ",
  "ラが〈わぐゎし〉なら、ビールが",
  "〈日本の味〉で\nあっても何ら不都",
  "合はないはずだ。実際、国産ビール",
  "の製造には米が一役かっている。",

// ここまでが「組見本用創作文『問わずがたりの洋酒外史』JIS 第１水準バージョン」です。

  "▷▶▷糸冬◁◀◁", // おしまいメッセージ
};

int char_table_size = sizeof(char_table) / sizeof(*char_table);


int main (void)
{
	int fd, n;

	// wiringPi 初期化：以下の４方法のいずれか１つのみ実行する
	if (wiringPiSetup() == -1) return 1;
//	if (wiringPiSetupGpio() == -1) return 1;
//	if (wiringPiSetupPhys() == -1) return 1;
//	if (wiringPiSetupSys() == -1) return 1;

	// 漢字 LCD シールド(π用)の初期化
	fd = knjlcdInit (KNJLCD_I2C_ADDR);  // file descriptor を返します。エラーの時は負の値になります。
	if (fd < 0) {
		return 1;
	}

	// 文字表を順番に表示させる
	for (n = 0; n < char_table_size; n++) {
		// 画面クリア
		knjlcdClear (fd);

		// 文字列を漢字 LCD シールド(π用)に転送する
		knjlcdPuts (fd, char_table[n]);

//		// 適当な待ち時間を入れる。
//		delay (2000);

		getchar();  // Enterキー入力待ち
	}

	return 0;
}

